from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.error import Error
from ...models.unit_temperature_conversion import UnitTemperatureConversion
from ...models.unit_temperature_format import UnitTemperatureFormat
from ...types import Response


def _get_kwargs(
    output_format: UnitTemperatureFormat,
    src_format: UnitTemperatureFormat,
    value: float,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/unit/conversion/temperature/{src_format}/{output_format}".format(
        client.base_url, output_format=output_format, src_format=src_format
    )  # noqa: E501
    if value is not None:
        if "?" in url:
            url = url + "&value=" + str(value)
        else:
            url = url + "?value=" + str(value)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[Any, UnitTemperatureConversion, Error]]:
    if response.status_code == 200:
        response_200 = UnitTemperatureConversion.from_dict(response.json())
        return response_200
    if response.status_code == 400:
        response_4XX = Error.from_dict(response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error.from_dict(response.json())
        return response_5XX
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[Any, UnitTemperatureConversion, Error]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    output_format: UnitTemperatureFormat,
    src_format: UnitTemperatureFormat,
    value: float,
    *,
    client: Client,
) -> Response[Union[Any, UnitTemperatureConversion, Error]]:
    kwargs = _get_kwargs(
        output_format=output_format,
        src_format=src_format,
        value=value,
        client=client,
    )

    response = httpx.get(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    output_format: UnitTemperatureFormat,
    src_format: UnitTemperatureFormat,
    value: float,
    *,
    client: Client,
) -> Optional[Union[Any, UnitTemperatureConversion, Error]]:
    """Convert a temperature unit value to another temperature unit value. This is a nice endpoint to use for helper functions."""  # noqa: E501

    return sync_detailed(
        output_format=output_format,
        src_format=src_format,
        value=value,
        client=client,
    ).parsed


async def asyncio_detailed(
    output_format: UnitTemperatureFormat,
    src_format: UnitTemperatureFormat,
    value: float,
    *,
    client: Client,
) -> Response[Union[Any, UnitTemperatureConversion, Error]]:
    kwargs = _get_kwargs(
        output_format=output_format,
        src_format=src_format,
        value=value,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    output_format: UnitTemperatureFormat,
    src_format: UnitTemperatureFormat,
    value: float,
    *,
    client: Client,
) -> Optional[Union[Any, UnitTemperatureConversion, Error]]:
    """Convert a temperature unit value to another temperature unit value. This is a nice endpoint to use for helper functions."""  # noqa: E501

    return (
        await asyncio_detailed(
            output_format=output_format,
            src_format=src_format,
            value=value,
            client=client,
        )
    ).parsed



from enum import Enum


class UnitTimeFormat(str, Enum):
    """The valid types of time unit formats."""  # noqa: E501

    """# <https://en.wikipedia.org/wiki/Second> """  # noqa: E501
    SECOND = "second"
    """# <https://en.wikipedia.org/wiki/Minute> """  # noqa: E501
    MINUTE = "minute"
    """# <https://en.wikipedia.org/wiki/Hour> """  # noqa: E501
    HOUR = "hour"
    """# <https://en.wikipedia.org/wiki/Day> """  # noqa: E501
    DAY = "day"
    """# <https://en.wikipedia.org/wiki/Week> """  # noqa: E501
    WEEK = "week"
    """# <https://en.wikipedia.org/wiki/Year> """  # noqa: E501
    YEAR = "year"
    """# <https://en.wikipedia.org/wiki/Julian_year> """  # noqa: E501
    JULIAN_YEAR = "julian_year"
    """# <https://en.wikipedia.org/wiki/Gregorian_calendar> """  # noqa: E501
    GREGORIAN_YEAR = "gregorian_year"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UnitVelocityFormat(str, Enum):
    """The valid types of velocity unit formats."""  # noqa: E501

    """# <https://en.wikipedia.org/wiki/Metre_per_second> """  # noqa: E501
    METERS_PER_SECOND = "meters_per_second"
    """# <https://en.wikipedia.org/wiki/Foot_per_second> """  # noqa: E501
    FEET_PER_SECOND = "feet_per_second"
    """# <https://en.wikipedia.org/wiki/Miles_per_hour> """  # noqa: E501
    MILES_PER_HOUR = "miles_per_hour"
    """# <https://en.wikipedia.org/wiki/Kilometres_per_hour> """  # noqa: E501
    KILOMETERS_PER_HOUR = "kilometers_per_hour"
    """# <https://en.wikipedia.org/wiki/Knot_(unit)> """  # noqa: E501
    KNOT = "knot"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UnitDataTransferRateFormat(str, Enum):
    """The valid types of data transfer unit formats."""  # noqa: E501

    """# <https://en.wikipedia.org/wiki/Byte> """  # noqa: E501
    BYTES_PER_SECOND = "bytes_per_second"
    """# <https://en.wikipedia.org/wiki/Byte#Multiple-byte_units> """  # noqa: E501
    EXABYTES_PER_SECOND = "exabytes_per_second"
    """# <https://en.wikipedia.org/wiki/Bit> """  # noqa: E501
    BITS_PER_SECOND = "bits_per_second"
    """# <https://en.wikipedia.org/wiki/Exabit> """  # noqa: E501
    EXABITS_PER_SECOND = "exabits_per_second"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UnitForceFormat(str, Enum):
    """The valid types of force unit formats."""  # noqa: E501

    """# <https://en.wikipedia.org/wiki/Newton_(unit)> """  # noqa: E501
    NEWTON = "newton"
    """# <https://en.wikipedia.org/wiki/Pound_(force)> """  # noqa: E501
    POUND = "pound"
    """# <https://en.wikipedia.org/wiki/Dyne> """  # noqa: E501
    DYNE = "dyne"
    """# <https://en.wikipedia.org/wiki/Kilogram-force> """  # noqa: E501
    KILOPOND = "kilopond"
    """# <https://en.wikipedia.org/wiki/Poundal> """  # noqa: E501
    POUNDAL = "poundal"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class UnitMagneticFieldStrengthFormat(str, Enum):
    """The valid types of magnetic field strength unit formats."""  # noqa: E501

    """# <https://en.wikipedia.org/wiki/Tesla_(unit)> """  # noqa: E501
    TESLA = "tesla"
    """# <https://en.wikipedia.org/wiki/Gauss_(unit)> """  # noqa: E501
    GAUSS = "gauss"

    def __str__(self) -> str:
        return str(self.value)
